/**
 * Tiny WMS SVG client
 * 
 * @licence  MIT licence
 * @author Olivier Courtin <courtin@cartoweb.org>
 * @version $Id$
 */

/*en cours de test*/
var numChecked = [];

// svg doc pointer
var svgdoc;


// wms object from GetCapabilities
var wms_struct;

// current cheked wms layers (first one on bottom's stack and so on)
var checked_layers = [];

// flag  initial layer retrieve
var flag_wms_initial_layer = true;

// flag to remind when a select process as began (after a mousedown in map area)
var select_on;

//flag to remind which action is currently checked
var action_checked=3;

// initial select x,y position
var select_x;
var select_y;


// to avoid tilts with dummy's users...
var flag_in_check = false;

// Map viewbox x, y, width, height 
var viewbox_x;
var viewbox_y;
var viewbox_w;
var viewbox_h;

// Map viewbox ratio (i.e how many CRS unit per pixel)
var viewbox_ratio;

// N,S,E,W selected coords in CRS unit (cf GetLatLonBbox)
var crs_north;
var crs_south;
var crs_east;
var crs_west;

// current bbox on zoom mode
var zoom_bbox = 0;

// current viewBoxColl (used for geometryCollection case, give the appropriate viewbox)
var viewBoxColl;

// main SVG width, height
//var screen_width = 640;
//var screen_height = 450;
var screen_width = 450;
var screen_height = 350;
// map origin x, y, width, height (from main SVG)
var map_x = 108;
var map_y = 33;
var map_width = screen_width-map_x;
var map_height = screen_height-map_y;

// map margin width, height (from main SVG)
var map_x_margin = 3;
var map_y_margin = 3;

//Zoom_scale_init used or not
var zoom_scale_init_used=false;
//Variable used to select geometries with associated metadata.
var geomCollSelection=false;

/***************** INITIALIZATION FUNCTION ***************/

/**
* wms_init(evt)
* Initialization function
* To init cartographic interface
*/
// wms_init(evt) {{{
function wms_init(evt)
{
 
 svgdoc = evt.target.ownerDocument;
 //hideDebugZone();
 var query = wms_server + "?SERVICE=WMS&REQUEST=GetCapabilities";
 getURL(query, capabilities_callback); 

	
}
// }}}

/******************** DEBUG FUNCTIONS ******************/

function setDebugZone(chaine)
{

 showDebugZone();
 debugzone = svgdoc.getElementById("debugzone");
  	
 var children = debugzone.getChildNodes();
 for (var i=0;i<children.length;i++)
 {
  	if (children.item(i).getNodeType() == 3) // text node
  	{
   	children.item(i).setNodeValue(chaine);
  	}
 }
}

function hideDebugZone()
{
 debugzone = svgdoc.getElementById("debugzone");
 debugzone.setAttribute("visibility", "hidden");
}

function showDebugZone()
{
 debugzone = svgdoc.getElementById("debugzone");
 debugzone.setAttribute("visibility", "visible");
 //debugzone.setVisible(true);
}



/***************** GET CAPABILITIES *******************/

/**
* capabilities_callback(request)
* Fonction appele lors de l'initialisation, elle permet d'appeler les fonctions ncessaires 
* au traitement du fichier get_capabilities.xml et  l'affichage des layers.
*/

// capabilities_callback(request) {{{
function capabilities_callback(request){
 if(!request.success)
  return false;
 
 wms_struct = capabilities_parser(request);
 layers_set();
 layers_get(0, layers_callback);
}
// }}}

/**
* capabilities_parser(request)
* Fonction analysant le fichier get_capabilities.xml pour en tirer les informations
* ncessaires au traitement de l'information gographique.
*/
// capabilities_parser(request) {{{
 // TODO Document result structure
function capabilities_parser(request)
{
	 var result = new Object();
	var doc = parseXML(request.content);
	 var root = doc.getElementsByTagName("WMS_Capabilities").item(0);
	 var capa = root.getElementsByTagName("Capability").item(0);
	 var request = capa.getElementsByTagName("Request").item(0);
	 var get_map = request.getElementsByTagName("GetMap").item(0);


	 // Format retrieve 
	 var format = get_map.getElementsByTagName("Format");
	 result.format = new Array();
	 for(i=0 , max=format.length ; i<max ; i++)
 	{
  		format.item(i).firstChild.normalize();
  		result.format.push(format.item(i).firstChild.nodeValue);
 	}

	 // URL GetMap GET retrieve 
	 var dcptype = get_map.getElementsByTagName("DCPType").item(0);
	 var http = dcptype.getElementsByTagName("HTTP").item(0);
	 var get = http.getElementsByTagName("Get").item(0);
	 var onl_res = get.getElementsByTagName("OnlineResource").item(0);
	 result.url_getmap = onl_res.getAttribute("xlink:href");


	 // Layers retrieve 
	 var layer_root = root.getElementsByTagName("Layer").item(0);
	 var layers = layer_root.getElementsByTagName("Layer");
	 result.layers = new Array();
	 for(i=0 , max=layers.length ; i<max ; i++)
 	{
  		var name  = layers.item(i).getElementsByTagName("Name").item(0);

		  // Some Layers could be nameless (cf WMS spec)
  		if(name != null){

	   name.firstChild.normalize();
	
	   var title = layers.item(i)
	    .getElementsByTagName("Title").item(0);
	   title.firstChild.normalize();
	
	   var bbox = layers.item(i)
	    .getElementsByTagName("BoundingBox").item(0);
	
		//contains the name, title, crs and bbox layer informations 
  
	   	result.layers.push({name:name.firstChild.nodeValue,
	     title:title.firstChild.nodeValue,
	     crs:bbox.getAttribute("CRS"),
	     minx:bbox.getAttribute("minx"),
	     miny:bbox.getAttribute("miny"),
	     maxx:bbox.getAttribute("maxx"),
	     maxy:bbox.getAttribute("maxy")
     	});
  }
 }

 return result;
}
// }}}



/************* LAYER NAME AND CHECKBOX ***********/

/**
* layers_set()
* Fonction affichant textuellement les diffrents layers (couches), prsents dans le
* fichier get_capabilities.xml, ainsi qu'une checkbox, dans l'lment SVG correspondant.
* (cf. wms_client.php, voir l'lment 'id="layers"').
*/
// layers_set() {{{
function layers_set()
{
 for(i=0 , max=wms_struct.layers.length ; i < max ; i++){

//To hidden the geometry collection table
 if(wms_struct.layers[i]["name"]!=wms_geomCollection_table)
 {
  	 
	 //Checkbox element
  var rect = svgdoc.createElement("rect");
  rect.setId("layer_" + i);
  rect.setAttribute("id", "layer_" + i);
  rect.setAttribute("x", "+7");
  rect.setAttribute("y", (10 * i) + 30);
  rect.setAttribute("width", 6);
  rect.setAttribute("height", 6);
  rect.setAttribute("fill", "#fff");
  rect.setAttribute("stroke-width", "1");
  rect.setAttribute("stroke", "#999");
  rect.setAttribute("onclick", "layers_click(evt);");

  svgdoc.getElementById("layers").appendChild(rect);
  
   //Displays a red rectangle when a layer is clicked.    
  var checked = svgdoc.createElement("rect");
  checked.setId("checked_" + i);
  checked.setAttribute("id", "checked_" + i);
  checked.setAttribute("x", "+7.3");
  checked.setAttribute("y", (10.3 * i) + 30);
  checked.setAttribute("width", 4);
  checked.setAttribute("height", 4);
  checked.setAttribute("fill", "#f66");
  checked.setAttribute("stroke-width", "0");
  checked.setAttribute("visibility", "hidden");
  checked.setAttribute("pointer-events", "none");

  svgdoc.getElementById("layers").appendChild(checked);

 //Displays each layer name 
  var text = svgdoc.createElement("text");
  text.setAttribute("x", "+17");
  text.setAttribute("y", (10 * i) + 38);
  text.setAttribute("fill", "#555");
  text.setAttribute("font-size","8");
  var title;
  if (i != 0)
  {
     title = svgdoc.createTextNode(wms_struct.layers[i]["title"]);
  }
  else
  {
     title = svgdoc.createTextNode(wms_struct.layers[i]["title"]);
  }
  text.appendChild(title);

  svgdoc.getElementById("layers").appendChild(text);
 		}
	}
}
// }}}

/************* CLICK ON LAYER CHECKBOX *************/
/**
* layers_click(evt)
* Cette fonction est appele lorsque l'utilisateur veut afficher ou enlever 
* une couche en cliquant sur une checkbox. Modification de la checkbox concerne (active ou non).
* Appel de la fonction layers_get(....) pour l'affichage graphique.
*/
// layers_click(evt) {{{
function layers_click(evt)
{
 if(flag_in_check == true || svgdoc.getElementById("metadata_view")!=null)

  return;

 flag_in_check = true;

 //replaces in "layer_layerName" 'layer_' by an empty string
 var wms_layer = evt.target.id.replace(/layer_/,"");
 var checked = svgdoc.getElementById("checked_" + wms_layer);

 wms_layers = [wms_layer];
 
 if(checked.getAttribute("visibility") == "hidden"){
  layers_get(0, layers_callback);
  	
 } else {

  var layer = svgdoc.getElementById("wms_layer_" + wms_layer);
  svgdoc.getElementById("wms_map").removeChild(layer);

  checked.setAttribute("visibility", "hidden");
  
  for(i=0 , max=checked_layers.length ; i < max ; i++){
   if(checked_layers[i] == wms_layer)
    checked_layers.splice(i, 1);
  }

  flag_in_check = false;
 }
}
// }}}

/************* GET MAP OR LAYER (WMS GetMap or GetLayer) *****************/
/**
* layers_get(bbox,callback)
* Cette fonction va constituer la requte ncessaire en fonction des actions de l'utilisateur, zoom avant,
* zoom arrire, affichage initial, slection etc...
*/
//  layers_get(bbox, callback) {{{
function layers_get(layers_bbox, callback)
{
 svgdoc.getElementById("loading").setAttribute("visibility", "visible");

 //Initial call. Displays all the layers defined in the file config.js 
 if(flag_wms_initial_layer){
  var name = "";

  var minx = 0;
  var maxx = 0;
  var miny = 0;
  var maxy = 0;


  for(i=0, max=wms_layers.length ; i < max ; i++) {
   name += wms_struct.layers[wms_layers[i]]["name"];
   if((i + 1) < max) name += ",";
 
   svgdoc.getElementById("checked_" + wms_layers[i])
    .setAttribute("visibility", "visible");

   if(i == 0){
    minx = wms_struct.layers[wms_layers[i]]["minx"];
    miny = wms_struct.layers[wms_layers[i]]["miny"];
   } else {

    minx = Math.min(
     wms_struct.layers[wms_layers[i]]["minx"]
     , minx
     );

    miny = Math.min(
     wms_struct.layers[wms_layers[i]]["miny"]
     , miny
     );
   }

   maxx = Math.max(
    wms_struct.layers[wms_layers[i]]["maxx"]
    , maxx
    );

   maxy = Math.max(
    wms_struct.layers[wms_layers[i]]["maxy"]
    , maxy
    );
   
  }
 
 } 
 else{//Displays one layer (when checkbox is used)
  var name = wms_struct.layers[wms_layers[0]]["name"];
 }

 //Determines the appropriate bounding box 
 if(flag_wms_initial_layer && layers_bbox == 0){
  bbox = minx + "," + miny + "," + maxx + "," + maxy;
  viewBoxColl = bbox;//Initial application viewbox
 } else if(layers_bbox == 0) {
  if(zoom_bbox == 0){
   bbox = wms_struct.layers[wms_layers[0]]["minx"] + ","
    + wms_struct.layers[wms_layers[0]]["miny"] + ","
    + wms_struct.layers[wms_layers[0]]["maxx"] + ","
    + wms_struct.layers[wms_layers[0]]["maxy"];
   } else {
    bbox = zoom_bbox;
   }
 } else {
  bbox = layers_bbox;
 }

 // FIXME : really ?
 var crs = wms_struct.layers[wms_layers[0]]["crs"];

 if(geomCollSelection && action_checked==5)
	name = wms_geomCollection_table;//defined in config.js
	
//http request constitution
 var query = wms_struct.url_getmap + "?SERVICE=WMS"
  + "&LAYERS=" + name 
  + "&VERSION=1.3.0"
  + "&TRANSPARENT=TRUE"
  + "&STYLES="
  + "&FORMAT=image/svg%2Bxml"
  + "&WIDTH=" + map_width 
  + "&HEIGHT=" + map_height
  + "&BBOX=" + bbox 
  + "&CRS=" + crs ;

 if(flag_wms_initial_layer && !geomCollSelection){
query += "&REQUEST=GetMap";
}else if(flag_wms_initial_layer && geomCollSelection){
query += "&VBOXCOLL=" + viewBoxColl;//To select geometries with metadata in the bbox zoom




  query += "&REQUEST=GetSelection";
} else {
  query += "&REQUEST=GetLayer";
}





 getURL(query, callback); 
}
// }}}

/**
* layers_callback(request)
* Cette fonction traite la requte constitue par la fonction prcdente.
* Analyse du fichier xml produit par l'appel  GetMap, GetSelection ou GetLayer.
*/
// layers_callback(request) {{{
function layers_callback(request)
{
 if(!request.success)
  return false;
 
 //var toto = new String(request.content);
  //setDebugZone(toto);
  var new_node = parseXML(request.content, svgdoc);
  var map = svgdoc.getElementById("wms_map");
  if(flag_wms_initial_layer){
         var result = svgdoc.getElementById("result");
         result.removeChild(map);
         result.appendChild(new_node);
	     result.lastChild.setId("wms_map");
         result.lastChild.setAttribute("id", "wms_map");
         result.lastChild.setAttribute("x", map_x_margin);
         result.lastChild.setAttribute("y", map_y_margin);
         result.lastChild.setAttribute("preserveAspectRatio","xMinYMin meet");
  var layer_g = result.getElementsByTagName("g");
  for(i=0 , max = layer_g.length ; i < max ; i++) {

 //A REVOIR... 
   if (action_checked==5 && geomCollSelection){
        checked_layers = [];
        for (i=0; i<numChecked.length; i++)
                checked_layers.push(numChecked[i]);
        }
   //...
if (wms_layers[i]!=null){//GeometryCollection case.
   layer_g.item(i).setId("wms_layer_" 
    + wms_layers[i]);
   layer_g.item(i).setAttribute("id", "wms_layer_" 
    + wms_layers[i]);
   var checked = svgdoc.getElementById("checked_" 
    + wms_layers[i]);
   checked.setAttribute("visibility", "visible");
   checked_layers.push(wms_layers[i]);
	}    
  }

  flag_wms_initial_layer = false;
  select_init();
	
  
  
  } else {
         map.appendChild(new_node);
    	 map.lastChild.setId("wms_layer_" + wms_layers[0]);
         map.lastChild.setAttribute("id", "wms_layer_" + wms_layers[0]);
 
  var checked = svgdoc.getElementById("checked_" + wms_layers[0]);
  checked.setAttribute("visibility", "visible");
  checked_layers.push(wms_layers[0]);
  }
  
 flag_in_check = false;
 svgdoc.getElementById("loading").setAttribute("visibility", "hidden");
}
// }}}


/************** GET LATITUDE LONGITUDE BBOX *******************/

// latlonbbox_click() {{{
function latlonbbox_click()
{
 // begin to check if something was selected
 if(select_on != 2 || action_checked!=2 ||
  (crs_north == 0 && crs_south == 0 
   && crs_east == 0 && crs_west == 0) )
  return;
  
 // TODO use url_getlatlonbbox instead...
 var query = wms_struct.url_getmap;

 query += "?SERVICE=WMS&REQUEST=GetLatLonBbox&VERSION=1.3.0"
  + "&BBOX=" + crs_west + "," + crs_south 
  + "," + crs_east + "," + crs_north;


 if(wms_deg_min_sec)
  query += "&DEG_MIN_SEC=TRUE";
  

 // FIXME really ?
 query += "&CRS=" + wms_struct.layers[0]["crs"];

 getURL(query, latlonbbox_callback);

 
 
}
// }}}

// latlonbbox_callback(request) {{{
function latlonbbox_callback(request){
	if(!request.success) 
	return false;
  	// parse XML result set
 var doc = parseXML(request.content);
 var root = doc.getElementsByTagName("EX_GeographicBoundingBox").item(0);
 var iso_north = root.getElementsByTagName("northBoundLatitude").item(0).firstChild.nodeValue;
 var iso_south = root.getElementsByTagName("southBoundLatitude").item(0).firstChild.nodeValue;
 var iso_east = root.getElementsByTagName("eastBoundLongitude").item(0).firstChild.nodeValue;
 var iso_west = root.getElementsByTagName("westBoundLongitude").item(0).firstChild.nodeValue;


 // Assign result to SVG iso_result box
 //svgdoc.getElementById("iso_n").firstChild.nodeValue=iso_north;
 //svgdoc.getElementById("iso_s").firstChild.nodeValue=iso_south;
 //svgdoc.getElementById("iso_e").firstChild.nodeValue=iso_east;
 //svgdoc.getElementById("iso_w").firstChild.nodeValue=iso_west;

 //svgdoc.getElementById("iso_result").setAttribute("visibility", "visible");
 //svgdoc.getElementById("submit_result").setAttribute("visibility","visible");
 //svgdoc.getElementById("calc_latlon").setAttribute("visibility","hidden");
submit_coord_values(iso_north,iso_east,iso_south,iso_west);
	
}
// }}}




/************** MAP SELECT **************/

// select_init() {{{
function select_init()
{
 var viewBox = svgdoc.getElementById("wms_map").getAttribute("viewBox");
 var params = viewBox.split(/\s*,\s*|\s+/);
 viewbox_x = parseFloat( params[0] );
 viewbox_y = parseFloat( params[1] );
 viewbox_w = parseFloat( params[2] );
 viewbox_h = parseFloat( params[3] );

 viewbox_ratio = Math.max(
    (viewbox_w / map_width), 
    (viewbox_h / map_height)
    );

 screen_ratio = Math.max(
    (screen_width / getInnerWidth()), 
    (screen_height / getInnerHeight())
    );

 select_on = 0;
 
 var rect = svgdoc.createElement("rect");
 rect.setAttribute("id", "select");
 rect.setAttribute("x", "0");
 rect.setAttribute("y", "0");
 rect.setAttribute("width", "0");
 rect.setAttribute("height", "0");
 rect.setAttribute("fill", "none");
 rect.setAttribute("stroke", "black");
 rect.setAttribute("stroke-width", "2");
 rect.setAttribute("opacity", "1");

 if(svgdoc.getElementById("select")!=null){
        var recup = svgdoc.getElementById("select");
        svgdoc.getElementById("result").removeChild(recup);
 }

 svgdoc.getElementById("result").appendChild(rect);
}
// }}}


/*********** MOUSE MOVE AND CLICK ****************/
// select_mousedown() {{{
function select_mousedown()
{
 if(action_checked==3) return;

 // screen could have been resized...
 screen_ratio = Math.max(
    (screen_width / getInnerWidth()), 
    (screen_height / getInnerHeight())
    );

 select_flush();
 select_on = 1;
}
// }}}

// select_mousemove() {{{
function select_mousemove()
{
  if(select_on != 1 || action_checked==3) return;
 
 var current_x = ((evt.clientX * screen_ratio) - map_x);
 var current_y = ((evt.clientY * screen_ratio) - map_y);
 var width = Math.abs(current_x - select_x);
 var height = Math.abs(current_y - select_y);

 var select = svgdoc.getElementById("select");
 select.setAttribute("width", width);
  select.setAttribute("height", height);

 if((current_x - select_x) < 0){
   select.setAttribute("x", current_x);

  crs_west = Math.round(
   ((current_x - map_x_margin) * viewbox_ratio)
   + viewbox_x
   );

  crs_east = Math.round(
   ((current_x - map_x_margin + width) * viewbox_ratio)
   + viewbox_x
   );

 } else {
   select.setAttribute("x", select_x);

  crs_west = Math.round(
   ((select_x - map_x_margin) * viewbox_ratio)
   + viewbox_x
   );

  crs_east = Math.round(
   ((select_x - map_x_margin + width) * viewbox_ratio)
   + viewbox_x
   );
 }

 
 // remember that y SVG axis is reverse one...

 if((current_y - select_y) < 0){
  select.setAttribute("y", current_y);

  crs_north = Math.round(
   - ((current_y - map_y_margin) * viewbox_ratio)
   - viewbox_y
   );

  crs_south = Math.round(
   - ((current_y - map_y_margin + height) * viewbox_ratio)
   - viewbox_y
   );
 } else {
  select.setAttribute("y", select_y);

  crs_north = Math.round(
   - ((select_y - map_y_margin) * viewbox_ratio)
   - viewbox_y
   );

  crs_south = Math.round(
   - ((select_y - map_y_margin + height) * viewbox_ratio)
   - viewbox_y
   );
 }
}
// }}}

// select_mouseup() {{{
function select_mouseup()
{
	 select_on = 2;
 	if(action_checked==1)
    	zoom_foreward();

 	if(action_checked==2){
    	latlonbbox_click();
    	
    }


}
// }}}

/****************** FLUSH EVENT **********************/
// select_flush() {{{
function select_flush()
{
	 select_on = 0;
	
	 //if(action_checked!=3 || zoom_backward_used){
	 //svgdoc.getElementById("iso_result").setAttribute("visibility", "hidden");
	 //svgdoc.getElementById("submit_result").setAttribute("visibility","hidden");
	 //svgdoc.getElementById("calc_latlon").setAttribute("visibility","visible");
	
	 //window.parent.document.getElementById("geomName").setAttribute("value","");
	 //}
 	/*if ((crs_north == 0 && crs_south == 0 && crs_east == 0 && crs_west == 0)){
	  /*window.parent.document.getElementById("northvalue").setAttribute("value","");
	  window.parent.document.getElementById("southvalue").setAttribute("value","");
	  window.parent.document.getElementById("eastvalue").setAttribute("value","");
	  window.parent.document.getElementById("westvalue").setAttribute("value","");*/
	//}

	//To delete the metadata svg list 
 	 if (svgdoc.getElementById("metadata_view")!=null){
		if((action_checked==4 || action_checked==5) && !zoom_scale_init_used)
		{}
		else
		svgdoc.getElementById("metadata").removeChild(svgdoc.getElementById("metadata_view"));
 	}
  
	 select_x = ( (evt.clientX  * screen_ratio) - map_x);
 	select_y = ( (evt.clientY * screen_ratio) - map_y);

	  var select = svgdoc.getElementById("select");
	  select.setAttribute("x", select_x);
	 select.setAttribute("y", select_y);
	 select.setAttribute("width", 0);
	 select.setAttribute("height", 0);

	 crs_north = 0;
	 crs_south = 0;
	 crs_east = 0;
	 crs_west = 0;

 	zoom_scale_init_used = false;
	geomCollSelection = false;
}
// }}}



/*********  MAP ZOOM *****************/

// zoom_foreward() {{{
function zoom_foreward(){
 if(select_on != 2 || checked_layers.length == 0 || 
  (crs_north == 0 && crs_south == 0 
   && crs_east == 0 && crs_west == 0) )
  return;
 
 wms_layers = checked_layers;
 checked_layers = [];

 var zoom_ratio_height = (crs_north - crs_south) / map_height;
 var zoom_ratio_width = (crs_east - crs_west) / map_width;
 var zoom_ratio = Math.max(zoom_ratio_height, zoom_ratio_width);
 crs_south = crs_north - (zoom_ratio * map_height); 
 crs_east = crs_west + (zoom_ratio * map_width); 
 
 zoom_bbox = crs_west + "," + crs_south 
  + "," + crs_east + "," + crs_north;

 select_flush();
 flag_wms_initial_layer = true;

 layers_get(zoom_bbox, layers_callback);
}
// }}}

// zoom_backward() {{{
function zoom_backward(){
}
//}}}
// zoom_scale_init() {{{
function zoom_scale_init(){
 zoom_scale_init_used=true;


 if(checked_layers.length == 0)
  return;
 // FIXME : need to check if backward really needed

 wms_layers = checked_layers;
 checked_layers = [];

 zoom_bbox = 0;
 select_flush();
 flag_wms_initial_layer = true;

 layers_get(0, layers_callback);
}
// }}}


/******** SUBMIT N,S,E,W TO THE FORMULAR ******/

//Submit n,s,e,w values to html formular.
function submit_coord_values(n,e,s,w){
  var idGeom;
	
  try{
  	var nb_rub=svgdoc.getElementById("rub_occ").getAttribute("index");
	//transfertLatLong(10,20,30);
  	if (parent.document.title=="Cartographie svg"){
  		var parentHTML=window.opener.document;
  	}else{
  		var parentHTML=window.parent.document;
  	}
    var west_value = parentHTML.getElementById("latlong_west_"+nb_rub);
  	//var w = svgdoc.getElementById("iso_w").firstChild.nodeValue;
  	west_value.setAttribute("value",w);
  	var east_value = parentHTML.getElementById("latlong_east_"+nb_rub);
  	//var e = svgdoc.getElementById("iso_e").firstChild.nodeValue;
  	east_value.setAttribute("value",e);
  	var south_value = parentHTML.getElementById('latlong_south_'+nb_rub);
  	//var s = svgdoc.getElementById("iso_s").firstChild.nodeValue;
  	south_value.setAttribute("value",s);
  	var north_value = parentHTML.getElementById('latlong_north_'+nb_rub);
  	//var n = svgdoc.getElementById("iso_n").firstChild.nodeValue;
  	north_value.setAttribute("value",n);
  	n = n.match(/[-]?[0-9]+[.,]?[0-9]?[0-9]?[0-9]?/);
  	w = w.match(/[-]?[0-9]+[.,]?[0-9]?[0-9]?[0-9]?/);
  	s = s.match(/[-]?[0-9]+[.,]?[0-9]?[0-9]?[0-9]?/);
  	e = e.match(/[-]?[0-9]+[.,]?[0-9]?[0-9]?[0-9]?/);
  	idGeom="N_"+n+"_E_"+e+"_S_"+s+"_W_"+w;
  	var nodeToChange = parentHTML.getElementById('geographic_identifier_'+nb_rub);//node to modify
  	nodeToChange.setAttribute("value",idGeom);
	
	//var nodeToChange = parentHTML.getElementById("geographic_identifier_back");//node to modify
   	//nodeToChange.setAttribute("value",idGeom);
	}catch(exception){
		alert( "Foncionnalite impossible sur firefox/mozilla");
	}
}
/********* EVENTS ON GEOMETRIES ***************/
//when user overlaps a geom
function overGeom(label)
{
	
	if(action_checked==3 && svgdoc.getElementById("etiquette")==null && label!=null){
	

	/*
	if(type==1){
	var over = svgdoc.getElementById("geom_"+num);
	} else {
	//the geometry is type of POINT or MULTIPOINT.
	var over = svgdoc.getElementById("point_"+num);
	}
	*/
	
	if (evt.target!=null){
	
		var idMeta = svgdoc.getElementById("metadata");
		if (evt.target.getAttribute("id")=="metadata")
		{
			evt.target.setAttribute("fill","deepskyblue");
		
		}
		else
		{
			evt.target.setAttribute("opacity","0.5");
		}
	        //current path pointed.
		
		//current mouse x,y site
		var screen_ratio = Math.max(
	    	(screen_width / getInnerWidth()), 
	    	(screen_height / getInnerHeight())
	    	);
		var current_x = ((evt.clientX * screen_ratio));
	 	var current_y = ((evt.clientY * screen_ratio));
		
		var where = svgdoc.getElementById("temporary_label");
		var etiquette = svgdoc.createElement("rect");
		where.appendChild(etiquette);
		
		//pointed geometry name
		var new_text = svgdoc.createElement("text");
	        var label_etiquette = svgdoc.createTextNode(label);
	        new_text.appendChild(label_etiquette);
	
		etiquette.setAttribute("id","etiquette");
		etiquette.setAttribute("x",current_x+5);
		etiquette.setAttribute("y",current_y-3);



		if (label!="")
		{
			etiquette.setAttribute("width",new_text.getComputedTextLength());
			etiquette.setAttribute("height","10");
		}
		else
		{
			etiquette.setAttribute("width","0");
			etiquette.setAttribute("height","0");
		}
	
		etiquette.setAttribute("fill","white");
		etiquette.setAttribute("stroke","#333");
		etiquette.setAttribute("stroke-width","0.5");
		etiquette.setAttribute("visibility","visible");
		
		where.appendChild(new_text);
		new_text.setAttribute("id","text_etiquette");
		new_text.setAttribute("x",current_x+10);
		new_text.setAttribute("y",current_y+5);
		new_text.setAttribute("font-familly","Verdana");
		new_text.setAttribute("font-size","8");
		new_text.setAttribute("fill","#555");
		}

	}

}

//overGeom(num) inverse
function outGeom()
{	
	if(action_checked==3){

	/*
	if (type==1){
	var out = svgdoc.getElementById("geom_"+num);
	} else {	 
	var out = svgdoc.getElementById("point_"+num);
	}
	*/

	//current path pointed.
	if (evt.target.getAttribute("id")=="metadata")
	{
		evt.target.setAttribute("fill","none");
		
	}
	else
	{
		evt.target.setAttribute("opacity","1");
	}
		
	var toDeleteEtiquette = svgdoc.getElementById("etiquette");
	var toDeleteText = svgdoc.getElementById("text_etiquette");
	var where = svgdoc.getElementById("temporary_label");
	where.removeChild(toDeleteEtiquette);
	where.removeChild(toDeleteText);
	}
}
//actions checked by users
function checkbox_action(item)
{
	action_checked=item;
	select_flush();
	for(i=1; i<=3; i++)//number of actions
	{
    	if(i!=item)
	  		svgdoc.getElementById("action_"+i).setAttribute("visibility","hidden");
	}
	svgdoc.getElementById("action_"+item).setAttribute("visibility","visible");
}
//To submit the geometry name to HTML formular
function click_on_geom(label,n,w,s,e){
	var nb_rub=svgdoc.getElementById("rub_occ").getAttribute("index");
	 if(action_checked==3 && label!=null){
	    try{
  			if (window.parent.document.title=="Cartographie svg"){
  				var parentHTML = window.opener.document;
			}else{
  				var parentHTML = window.parent.document;
  			}
  	       	var nodeToChange = parentHTML.getElementById("geographic_identifier_"+nb_rub);//node to modify
			if (nodeToChange!=null)
     			nodeToChange.setAttribute("value",label);
     		var nodeToChange = parentHTML.getElementById("geographic_identifier_back_"+nb_rub);//node to modify
     		if (nodeToChange!=null){
     			nodeToChange.setAttribute("value",label);
     		}
     	   	var west_value = parentHTML.getElementById("latlong_west_"+nb_rub);
  			west_value.setAttribute("value",w);
  			var east_value = parentHTML.getElementById("latlong_east_"+nb_rub);
  			east_value.setAttribute("value",e);
  			var south_value = parentHTML.getElementById("latlong_south_"+nb_rub);
  			south_value.setAttribute("value",s);
  			var north_value = parentHTML.getElementById("latlong_north_"+nb_rub);
  			north_value.setAttribute("value",n);
     	
  	       		//select_flush();
     		
		}catch(exception){
			alert( "Error");
		}
	}
}
//get all the geometries which have metadata associated.
function geomColl_selection()
{
	if(action_checked!=5 || (crs_north == 0 && crs_south == 0 && crs_east == 0 && crs_west == 0))
		return;
  	geomCollSelection = true;
  	flag_wms_initial_layer=true;

  	wms_layers = checked_layers;
	for(i=0; i<checked_layers.length; i++)
		numChecked[i]=checked_layers[i];
  	checked_layers = [];

	//Initial viewbox
  	//viewBoxColl = 547293 + "," + 1703210 + "," + 801423 + "," + 1997767;
  
  	if(zoom_bbox!=0)//If user has used zoom
		viewBoxColl = zoom_bbox;
  	var selected_bbox = crs_west + "," + crs_south + "," + crs_east + "," + crs_north;
	//Wait 500ms before to call 'selectedObjectsList()' function 
	//setTimeout("selectedObjectsList()", 500);
  	layers_get(selected_bbox,layers_callback);
}

//Display textual objects selection.
function selectedObjectsList()
{ 
	var meta = svgdoc.getElementsByTagName("path");
	var metaBis = svgdoc.getElementsByTagName("circle");
	var where = svgdoc.getElementById("metadata");

	//Y initial position of the first text object
	y_text = 50;
	num = 1;

	var new_g = svgdoc.createElement("g");
	new_g.setAttribute("id","metadata_view");

	where.appendChild(new_g);

	var rect = svgdoc.createElement("rect");
	rect.setAttribute("x","1");
	rect.setAttribute("width","200");
	rect.setAttribute("height","580");
	rect.setAttribute("fill","#dfd");
	rect.setAttribute("stroke","#333");

	var title = svgdoc.createElement("text");
	var title_name =  svgdoc.createTextNode("Fiches");
	title.appendChild(title_name);       
	title.setAttribute("x","80");
	title.setAttribute("y","20");
	title.setAttribute("font-size","10");

	var where_g = svgdoc.getElementById("metadata_view");
	where_g.appendChild(rect);
	where_g.appendChild(title);

	for(i=0; i<meta.length; i++){
		if (meta.item(i).getAttribute("id")=="metadata"){

		var split_string = meta.item(i).getAttribute("mdata").split(",");

			for (j=0; j<split_string.length; j++){

				var new_text = svgdoc.createElement("text");
                		var label_metadata = svgdoc.createTextNode(num+".  "+meta.item(i).getAttribute("name")+" : Fiche "+split_string[j]);
                		new_text.appendChild(label_metadata);
		
                		new_text.setAttribute("id","label_metadata");
                		new_text.setAttribute("x",5);
                		new_text.setAttribute("y",y_text);
                		new_text.setAttribute("font-familly","Verdana");
                		new_text.setAttribute("font-size","8");
                		new_text.setAttribute("fill","#333");
                		//new_text.setAttribute("text-decoration","underline");
                		new_text.setAttribute("onclick","displayMetadata('TEST')");
				
				where_g.appendChild(new_text);
				y_text+=25;
				num++;
			}

		
		/*
		var new_text = svgdoc.createElement("text");
       	var label_metadata = svgdoc.createTextNode(num+".  "+meta.item(i).getAttribute("name"));
        	new_text.appendChild(label_metadata);

		new_text.setAttribute("id","label_metadata");
	        new_text.setAttribute("x",10);
	        new_text.setAttribute("y",y_text);
	        new_text.setAttribute("font-familly","Verdana");
	        new_text.setAttribute("font-size","12");
	        new_text.setAttribute("fill","#333");
		//new_text.setAttribute("text-decoration","underline");
		new_text.setAttribute("onclick","displayMetadata('TEST')");

		where_g.appendChild(new_text);	

		y_text+=25;
		num++;
		*/
		}
	}

	for(i=0; i<metaBis.length; i++){
        	if (metaBis.item(i).getAttribute("id")=="metadata"){
	
		var split_string = metaBis.item(i).getAttribute("mdata").split(",");

                        for (j=0; j<split_string.length; j++){

                                var new_text = svgdoc.createElement("text");
                                var label_metadata = svgdoc.createTextNode(num+".  "+metaBis.item(i).getAttribute("name")+" : Fiche "+split_string[j]);
                                new_text.appendChild(label_metadata);

                                new_text.setAttribute("id","label_metadata");
                                new_text.setAttribute("x",5);
                                new_text.setAttribute("y",y_text);
                                new_text.setAttribute("font-familly","Verdana");
                                new_text.setAttribute("font-size","8");
                                new_text.setAttribute("fill","#333");
                                //new_text.setAttribute("text-decoration","underline");
                                new_text.setAttribute("onclick","displayMetadata('TEST')");

                                where_g.appendChild(new_text);
                                y_text+=25;
                                num++;
                        }


		/*	
	        var new_text = svgdoc.createElement("text");
	        var label_metadata = svgdoc.createTextNode(num+".  "+metaBis.item(i).getAttribute("name"));
	        new_text.appendChild(label_metadata);
	
	        new_text.setAttribute("id","label_metadata");
	        new_text.setAttribute("x",10);
	        new_text.setAttribute("y",y_text);
	        new_text.setAttribute("font-familly","Verdana");
	        new_text.setAttribute("font-size","12");
	        new_text.setAttribute("fill","#333");
		new_text.setAttribute("onclick","displayMetadata('TEST')");

	        where_g.appendChild(new_text);
	
	        y_text+=25;
	num++;
		*/
	        }
	    }
}

//Display geometry associated metadata when a geometry is clicked.
function displayMetadata(data)
{
	if(action_checked==5){
        alert("Repassez en mode 'info'\net cliquer sur une geometrie\npour acceder a ses infos");
        return;
    }

   if(data!=null){

    var tableau = data.split(",");

    if (tableau.length>1)
    {
		alert("Il y a plusieurs fiches associees a cet objet,\nfiche(s) de metadonnee num : "+data);
    
    }
    else {	
	try{
		//Execution on the HTML document, not on the SVG document !!!
		parent.window.open('','Interface_Cartographique_Syscolag','width=800,height=600,resizable=yes,scrollbars=yes');
 	}
	catch(err){
		alert("popup bloquee !");
	}
   }	
    }
    else
	alert("ERREUR : les donnees sont vides");
}

//Display an information for each action when the user overlaps the action icon
function actionInfo(information)
{

//setTimeout("actionInfo(information)",1500)

		var where = svgdoc.getElementById("temporary_info");
                var etiquette = svgdoc.createElement("rect");
                where.appendChild(etiquette);

                //pointed geometry name
                var new_text = svgdoc.createElement("text");
                var label_etiquette = svgdoc.createTextNode(information);
                new_text.appendChild(label_etiquette);

                etiquette.setAttribute("id","etiquette");
                etiquette.setAttribute("x",evt.clientX-10);
                etiquette.setAttribute("y",evt.clientY+10);
                etiquette.setAttribute("width",new_text.getComputedTextLength());
                etiquette.setAttribute("height","10");
                etiquette.setAttribute("fill","lightyellow");
                etiquette.setAttribute("stroke","#333");
                etiquette.setAttribute("stroke-width","0.5");
                etiquette.setAttribute("visibility","visible");

                where.appendChild(new_text);
                new_text.setAttribute("id","text_etiquette");
                new_text.setAttribute("x",evt.clientX-5);
                new_text.setAttribute("y",evt.clientY+17);
                new_text.setAttribute("font-familly","Verdana");
                new_text.setAttribute("font-size","8");
                new_text.setAttribute("fill","#555");
	
}

//Delete the information label
function deleteInfo()
{ 
  var toDeleteEtiquette = svgdoc.getElementById("etiquette");
  var toDeleteText = svgdoc.getElementById("text_etiquette");
  var where = svgdoc.getElementById("temporary_info");
  
  if (toDeleteEtiquette!= null && toDeleteText!=null){
  	where.removeChild(toDeleteEtiquette);
  	where.removeChild(toDeleteText);
  }
}
