<?php

/**
 *  Common WMS toolbox librairie 
 *
 *  @licence    MIT licence
 *  @author     Olivier Courtin <ol@cartoweb.org>
 *  @version    $Id: wms_lib.php,v 1.4 2004/11/12 03:28:58 ol Exp $
 */



/*
 * Log facilities, mostly for debug purpose
 */


// function wlog($msg) {{{
function wlog($msg){
    if(!WMS_LOG) return;

    $log = fopen(WMS_LOG_FILE, 'a+'); 
    fwrite($log, $msg."\n");
    fclose($log);
}
// }}}

function writeLog($msg){
	$handle = fopen("Err.txt",'a+');
	fwrite($handle,$msg);
	fclose($handle);
}


/*
 * Some usefull functions to handle PostgreSQL query
 */


// _db_query($sql) {{{
function _db_query($sql)
{
    $db_connect = sprintf("host=%s port=%s dbname=%s user=%s password=%s",
        WMS_PG_HOST, WMS_PG_PORT, WMS_PG_NAME, WMS_PG_USER, WMS_PG_PASS);

    // Once a connection is already open 
    // pg_connect don't really open
    // a new one (cf: man 7 pg_connect). 
    $db = @pg_connect($db_connect);
    if(!is_resource($db))
        user_error("No Postgis server connect", E_USER_ERROR); 

    $result = pg_query($db, $sql);
    if(!is_resource($result))
        user_error("SQL Error : $sql", E_USER_ERROR);

    return $result;
}
// }}}

// db_array($sql) {{{
function db_array($sql)
{
    $result = _db_query($sql);
    $max = pg_num_rows($result);    

    for($i=0, $array=array() ; $i < $max ; $i++)
        $array[] = pg_fetch_array($result, $i, PGSQL_ASSOC);

    return $array;
}
// }}}

function _db_query_mdweb($sql)
{
    
    $db_connect = sprintf("host=%s port=%s dbname=%s user=%s password=%s",
        WMS_PG_HOST_MDWEB, WMS_PG_PORT_MDWEB, WMS_PG_NAME_MDWEB, WMS_PG_USER_MDWEB, WMS_PG_PASS_MDWEB);
	
	/*writeLog(WMS_PG_HOST_MDWEB);
	writeLog(WMS_PG_PORT_MDWEB);
	writeLog(WMS_PG_NAME_MDWEB);
	writeLog(WMS_PG_USER_MDWEB);
	writeLog(WMS_PG_PASS_MDWEB);
	*/
	
	
    // Once a connection is already open 
    // pg_connect don't really open
    // a new one (cf: man 7 pg_connect). 
    $db = @pg_connect($db_connect);
    
    if(!is_resource($db))
        user_error("No Postgis server connect", E_USER_ERROR); 

	
    $result = pg_query($db, $sql);
    
    if(!is_resource($result))
        user_error("SQL Error : $sql", E_USER_ERROR);

    return $result;
}
// }}}

// db_array($sql) {{{
function db_array_mdweb($sql)
{
    $result = _db_query_mdweb($sql);
    $max = pg_num_rows($result);    
   
    for($i=0, $array=array() ; $i < $max ; $i++)
        $array[] = pg_fetch_array($result, $i, PGSQL_ASSOC);
   
   
    return $array;
}



// db_line($sql) {{{
function db_line($sql)
{
    $result = _db_query($sql); 
    if(pg_num_rows($result)!=0)//rows number of the sql query.
	    return pg_fetch_array($result, 0, PGSQL_ASSOC);
    else
	    return -1;//rows number = 0.
}
// }}}

// db_item($sql, $key) {{{
function db_item($sql, $key)
{
    $res = db_line($sql);
    if(empty($res[$key]))
        user_error("SQL Error : $sql", E_USER_ERROR);

    return $res[$key];
}
// }}}




/*
 * Kind of API on Postgis tables
 */



// layer_svg_path($layers, $bbox, $srid, $ratio) {{{
function layer_svg_path($layers, $bbox, $srid, $ratio)
{
            $data = array();
   foreach($layers as $layer){

        extract(layer_get_info($layer));

        switch($type){
        case 'POINT':
        case 'MULTIPOINT':
            $rel = 0;   // to have circle output with assvg
            break;
        case 'LINESTRING':
        case 'POLYGON':
        case 'MULTIPOLYGON':
        case 'MULTILINESTRING':
            $rel = 1;   // to have relative path with assvg 
            break;
		case 'GEOMETRY':
	    $rel = -1;  // temporary value waiting the next switch case.
	    break;
        default:
            $error_msg = "PostGIS geometry type not supported : " . $type;
            user_error($error_msg, E_USER_ERROR);
        }

        // SVG query on this layer
		/*
        $sql = "
            SELECT assvg(simplify($f_geometry_column, $ratio), $rel, 0)
            FROM $layer
            ";
*/

	//SQL query to retrieve name and svg path.
 $sql = "
            SELECT nom,n,w,s,e,style,assvg($f_geometry_column, $rel, 0)   FROM $layer
            ";

        // sub query test case
        
        $diff = array_diff($bbox, layer_get_bbox($layer));
        if(!empty($diff)){
            // only query on a sub layer area
            $sql .= " WHERE $layer.the_geom && ((GeomFromText('POLYGON(($bbox[2] $bbox[3], $bbox[0] $bbox[3], $bbox[0] $bbox[1], $bbox[2] $bbox[1], $bbox[2] $bbox[3]))',$srid)))";
        }
		//writeLog($sql);
        switch($type){
        case 'MULTIPOINT':
            $point = array();
            // AsSVG use comma separator for MULTIPOINT values
            foreach(db_array($sql) as $multipoint)
                foreach(explode(',', $multipoint['assvg']) as $assvg)
                    $point[] = array('assvg' => $assvg);
            $data[] = $point;
            $data[count($data) - 1]['path'] = $rel;
            $data[count($data) - 1]['rayon'] = $rayon;
            
            break;
        case 'POINT':
            $data[] = db_array($sql);
            $data[count($data) - 1]['path'] = $rel;
            $data[count($data) - 1]['rayon'] = $rayon;
            
            break;
		case 'GEOMETRY':
 		$nbData = count($data); 
 	    //get the geometry column (geometrycollection type) from the $layer
	    $sql_the_geom = "SELECT the_geom FROM $layer";
	   
	    foreach(db_array($sql_the_geom) as $id => $value){
		
		//get the geometry type of each $layer row.
		$sql_geom_type = "SELECT geometrytype('".$value['the_geom']."') as geom_type";
		$geomType = db_line($sql_geom_type);
	
		if ($geomType['geom_type']=="POINT" || $geomType['geom_type']=="MULTIPOINT"){
			//Type POINT or MULTIPOINT.
			$rel = 0;
			
			//get the name and the path of the geometry.
			$sql_circle = "SELECT id_geom,nom,n,w,s,e,style,assvg('".$value['the_geom']."', $rel, 0) FROM $layer";

        		// sub query test case
        		$diff = array_diff($bbox, layer_get_bbox($layer));
				if(!empty($diff)){
            				// only query on a sub layer area
            				$sql_circle .= " WHERE '".$value['the_geom']."' && ((GeomFromText(
                				'POLYGON(($bbox[2] $bbox[3],
                        			       	  $bbox[0] $bbox[3],
                        				  $bbox[0] $bbox[1],
                        				  $bbox[2] $bbox[1],
                        				  $bbox[2] $bbox[3]))'
                					  ,$srid))) 
                 				 	  AND $layer.the_geom='".$value['the_geom']."'";
        			}
				else {
					$sql_circle .= " WHERE $layer.the_geom='".$value['the_geom']."'";
				}
			$res = db_line($sql_circle);
			if ($res != -1){//if rows number of the sql query is different from 0.
			    $data[$nbData][] = $res;
			    $data[$nbData]['path'] = $rel;
			    $data[$nbData]['rayon'] = $rayon;
			}
		}
		else {
			//Type LINESTRING, POLYGON, MULTIPOLYGON or MULTILINESTRING.
			$rel = 1;

			//get the name and the path of the geometry.
			$sql_path = "SELECT id_geom,nom,n,w,s,e,style,assvg('".$value['the_geom']."', $rel, 0) FROM $layer";
						
        		// sub query test case
        		$diff = array_diff($bbox, layer_get_bbox($layer));
				if(!empty($diff)){
            				// only query on a sub layer area
            				$sql_path .= " WHERE '".$value['the_geom']."' && ((GeomFromText(
                			'POLYGON(($bbox[2] $bbox[3],
                        			  $bbox[0] $bbox[3],
                        			  $bbox[0] $bbox[1],
                        			  $bbox[2] $bbox[1],
                        			  $bbox[2] $bbox[3]))'
                				  ,$srid)))
                 				  AND $layer.the_geom='".$value['the_geom']."'";
                 				  
				}
				else{
					$sql_path .= " WHERE $layer.the_geom='".$value['the_geom']."'"; 
				}
			$res = db_line($sql_path);
			if ($res != -1){//if rows number of the sql query is different from 0.
			    $data[$nbData+1][] = $res;
                $data[$nbData+1]['path'] = $rel;
			}
	     }
	}
	break;        
default:
	        $data[] = db_array($sql);
            $data[count($data)-1]['path'] = $rel;
        }
    }
return $data;
}
// }}}

//metadata_search($id_geom,$layer){{{
function metadata_search($id_geom,$layer)
{  
   //$layer is the geometrycollection table name
   
   $result_md = array();
   $final_res = "";
   
   if ($id_geom!=null && $layer!=null){
      $sql = "SELECT sheet_id, code_observatoire FROM md_og WHERE id_geom = $id_geom";
      //writeLog($sql);
      $result_md[] = db_array_mdweb($sql);
   		
   }
   
   for($i=0; $i<count($result_md[0]); $i++){
	   	$final_res = $final_res.$result_md[0][$i]['code_observatoire']."_".$result_md[0][$i]['sheet_id']." ";
   }
   
   return $final_res;
}
// }}}

// layer_is_valid($layer_name) {{{
function layer_is_valid($layer_name)
{
    // TODO include_once and static $layers !

    include('config/layers.php');

    if(empty($layers[$layer_name]))
        return false;

    return true;
}
// }}}

// layer_get_bbox($layer_name) {{{
function layer_get_bbox($layer_name)
{
    $sql = "
        SELECT xmin(extent(the_geom)), ymin(extent(the_geom)),
    xmax(extent(the_geom)), ymax(extent(the_geom))
        FROM $layer_name
        ";

    $bbox = db_line($sql);
    foreach($bbox as $value)
        $result[] = round($value);

    return $result;
}
// }}}

// layer_get_info($layer_name) {{{
function layer_get_info($layer_name)
{
    $result = array();

    // TODO include_once and static $layers !
    include('config/layers.php');
    if(!empty($layers[$layer_name]['rayon']))
        $result['rayon'] = $layers[$layer_name]['rayon'];

    $sql = "
        SELECT srid, type, f_geometry_column 
        FROM geometry_columns 
        WHERE f_table_name='$layer_name'
        ";
    foreach(db_line($sql) as $key => $info)
        $result[$key] = $info;

    return $result;
}
// }}}



/*
 * Input data API and sanity check
 */


// input_get_format() {{{
function input_get_format()
{
    if(empty($_GET['FORMAT']))
        user_error("MissingFormat");

    switch($_GET['FORMAT'])
    {
        // At this time, only support SVG
    case 'image/svg+xml':
        break;

    default:
        user_error("InvalidFormat");
    }

    return $_GET['FORMAT'];
}
// }}}

// input_get_width() {{{
function input_get_width()
{
    if(empty($_GET['WIDTH']))
        user_error("MissingWeight");

    if(!preg_match("/^[0-9]+$/", $_GET['WIDTH']))
        user_error("InvalidWeight");

    return $_GET['WIDTH'];
}
// }}}

// input_get_height() {{{
function input_get_height()
{
    if(empty($_GET['HEIGHT']))
        user_error("MissingHeight");

    if(!preg_match("/^[0-9]+$/", $_GET['HEIGHT']))
        user_error("InvalidHeight");

    return $_GET['HEIGHT'];
}
// }}}

// input_get_layers() {{{
function input_get_layers()
{
    if(empty($_GET['LAYERS']))
        user_error("LayerNotDefined");

    $layers = explode(',', $_GET['LAYERS']);
    //TODO for GetLayer request, need to check if only one layer is set.

    foreach($layers as $layer)
        if(!layer_is_valid($layer))
            user_error("LayerNotDefined"); 

    return $layers;
}
// }}}

// input_get_styles() {{{
function input_get_styles()
{
    if(!isset($_GET['STYLES']))
        user_error("StyleNotDefined");

    // TODO : implement user wanted styles
    return null; 
}
// }}}

// input_get_srid() {{{
function input_get_srid()
{
    if(empty($_GET['CRS']))
        user_error("MissingCRS");

    if(!preg_match("/^[a-zA-Z]+[:]([0-9]+)$/", $_GET['CRS'], $srid))
        user_error("InvalidCRS");

    // TODO need to check if srid is available for all layers...

    return $srid[1];
}
// }}}

// input_get_bbox() {{{
function input_get_bbox()
{
    if(empty($_GET['BBOX']))
        user_error("MissingBbox");

    $bbox = explode(',', $_GET['BBOX']);
    if(sizeof($bbox) != 4)
        user_error("InvalidBbox");

    foreach($bbox as $key => $value){
        if(!preg_match("/^[0-9]+([.][0-9]+)?$/", $key))
            user_error("InvalidBbox");
        $bbox[$key] = $value;
	}

    // bbox must have a real surface...
    if(($bbox[0] - $bbox[2] == 0) || ($bbox[1] - $bbox[3] == 0))
        user_error("InvalidBbox");

    return $bbox;
}
// }}}

// input_get_viewboxColl() {{{
function input_get_viewboxColl()
{
    if(empty($_GET['VBOXCOLL']))
        user_error("MissingBbox");    

    $viewBoxColl = explode(',', $_GET['VBOXCOLL']);
   
    if(sizeof($viewBoxColl) != 4)
        user_error("InvalidBbox");

    foreach($viewBoxColl as $key => $value){
        if(!preg_match("/^[0-9]+([.][0-9]+)?$/", $key))
            user_error("InvalidBbox");
        $viewBoxColl[$key] = $value;
	}

    // viewBoxColl must have a real surface...
    if(($viewBoxColl[0] - $viewBoxColl[2] == 0) || ($viewBoxColl[1] - $viewBoxColl[3] == 0))
        user_error("InvalidBbox");
    
    return $viewBoxColl;
}
// }}}
// input_get_transparent() {{{
function input_get_transparent()
{
    if(!empty($_GET['TRANSPARENT']) && $_GET['TRANSPARENT'] == 'TRUE')
        return true;

    return false; // default value
}
// }}}

// input_get_bgcolor() {{{
function input_get_bgcolor()
{
    if(isset($_GET['BGCOLOR'])
        && preg_match("/^0x[0-9A-Fa-f]{6}$/", $_GET['BGCOLOR']))
        return "#" . substr($_GET['BGCOLOR'], 2);

    return "#ffffff"; // white is default color
}
// }}}

// input_get_version() {{{
function input_get_version()
{
    if(empty($_GET['VERSION']))
        user_error("MissingVersion");

    switch($_GET['VERSION'])
    {
        // At this time, only support 1.3.0 
    case '1.3.0':
        break;
    default:
        user_error("InvalidVersion");
    }

    return $_GET['VERSION'];
}
// }}}

// input_get_deg_min_sec() {{{
function input_get_deg_min_sec()
{
    if(!empty($_GET['DEG_MIN_SEC']) && $_GET['DEG_MIN_SEC'] == "TRUE")
        return true;

    return false; // default value 
}
// }}}




/*
 * Bbox management 
 */


// bbox_to_viewbox($bbox) {{{
function bbox_to_viewbox($bbox)
{
    return array(
        "xmin" => $bbox[0],
        "ymin" => $bbox[3] * -1,
        "width" => $bbox[2] - $bbox[0],
        "height" => $bbox[3] - $bbox[1]
        );
}
// }}}

// bbox_ratio($bbox, $width, $height) {{{
function bbox_ratio($bbox, $width, $height)
{
    $ratio = max( 
        (($bbox[2] - $bbox[0]) / $width),
        (($bbox[3] - $bbox[1]) / $height)
        );

    return $ratio / WMS_DP_SIMPLIFY;
}
// }}}




/*
 * Invproj minimalistic API 
 */


// proj_bbox2latlon($bbox, $srid, $deg_min_sec=false) {{{
function proj_bbox2latlon($bbox, $srid, $deg_min_sec=false)
{
    $sql = "SELECT get_proj4_from_srid($srid) AS datum";
    $datum = db_item($sql, 'datum');

    $invproj = WMS_INVPROJ_PATH; 

    return array_merge(
        _proj_point2latlon($bbox[0], $bbox[1], $datum, $invproj, $deg_min_sec),
        _proj_point2latlon($bbox[2], $bbox[3], $datum, $invproj, $deg_min_sec)
        );
}
// }}}

// _proj_point2latlon($x, $y, $datum, $invproj, $deg_min_sec) {{{
function _proj_point2latlon($x, $y, $datum, $invproj, $deg_min_sec)
{
    $result=array();

    // invproj man page cf : man 1 proj

    if($deg_min_sec == false){
        // decimal doted mode

        $res = `echo $x $y | $invproj -f %.4f $datum`; 
        $pattern = "/(-?[0-9]+[.][0-9]+)[^-0-9]+(-?[0-9]+[.][0-9]+)/";
        preg_match($pattern, $res, $match);

        $result[] = $match[1];
        $result[] = $match[2];

    } else {
        // deg min sec mode

        $res = `echo $x $y | $invproj $datum`; 
        $pattern = "/([0-9]+)[d]([0-9]+)[']([0-9]+)[^EW]+[EW]";
        $pattern .= "[^0-9]*([0-9]+)[d]([0-9]+)[']([0-9]+)/";
        preg_match($pattern, $res, $match);

        $result[] = $match[1] . '&#176; ' . $match[2]  . "' " . $match[3] . "\""; 
        $result[] = $match[4] . '&#176; ' . $match[5]  . "' " . $match[6] . "\"";
    }

    return $result;
}
// }}}


/*
 * Style CSS configuration handler
 */


// style_get_svg($style) {{{
function style_get_svg($style)
{
    // kind of tricky regex to convert CSS to inline SVG styles
    preg_match_all(
        "/[^#]*[#]([a-zA-Z0-9_]+)[^{]*[{]([^}]*)[}][.]*/",
        file_get_contents('config/styles.css', true),
        $matches
        );

    $id_style = array_search($style, $matches[1]);
    if($id_style === false)
        return false;

    // TODO do both in a single shoot ?
    $style = preg_replace(
        "/[:][[:space:]]*([^;]+)[;]/",
        "='\\1'", $matches[2][$id_style]
        );

    $style = preg_replace("/[[:space:]]+/", ' ', $style);

    return $style;
}
// }}}

// style_layer_default($layer) {{{
function style_layer_default($layer)
{
    // TODO include_once and static value

    include 'config/layers.php';

    $result = $layers[$layer]['style'];

    return $result;
}
// }}}


?>
